@@ -1,8 +1,8 @@
 /**************************************************************************
  *
- * Copyright 2014-2020 Valve Software
- * Copyright 2015-2020 Google Inc.
- * Copyright 2019-2020 LunarG, Inc.
+ * Copyright 2014-2022 Valve Software
+ * Copyright 2015-2022 Google Inc.
+ * Copyright 2019-2022 LunarG, Inc.
  * All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -27,7 +27,6 @@
 #include <string.h>
 #include <fstream>
 #include <iostream>
-#include <map>
 #include <sstream>
 #include <string>
 #include <sys/stat.h>
@@ -83,6 +82,25 @@ string GetEnvironment(const char *variable) {
     string output = buffer;
     delete[] buffer;
     return output;
+#elif defined(__ANDROID__)
+    string command = "getprop " + string(variable);
+    FILE *pPipe = popen(command.c_str(), "r");
+    if (pPipe != nullptr) {
+        char value[256];
+        fgets(value, 256, pPipe);
+        pclose(pPipe);
+
+        // Make sure its not an empty line and does not end in a newline
+        const auto str_len = strcspn(value, "\r\n");
+        if (str_len == 0) {
+            return "";
+        } else {
+            value[str_len] = '\0';
+            return string(value);
+        }
+    } else {
+        return "";
+    }
 #else
     return "";
 #endif
@@ -100,16 +118,17 @@ VK_LAYER_EXPORT const SettingsFileInfo *GetLayerSettingsFileInfo() { return &lay
 // as a filename. If successful, return file handle, otherwise stdout
 VK_LAYER_EXPORT FILE *getLayerLogOutput(const char *option, const char *layer_name) {
     FILE *log_output = NULL;
-    if (!option || !strcmp("stdout", option))
+    if (!option || !strcmp("stdout", option)) {
         log_output = stdout;
-    else {
+    } else {
         log_output = fopen(option, "w");
         if (log_output == NULL) {
-            if (option)
+            if (option) {
                 std::cout << std::endl
                           << layer_name << " ERROR: Bad output filename specified: " << option << ". Writing to STDOUT instead"
                           << std::endl
                           << std::endl;
+            }
             log_output = stdout;
         }
     }
@@ -117,7 +136,7 @@ VK_LAYER_EXPORT FILE *getLayerLogOutput(const char *option, const char *layer_na
 }
 
 // Map option strings to flag enum values
-VK_LAYER_EXPORT VkFlags GetLayerOptionFlags(string option, std::unordered_map<string, VkFlags> const &enum_data,
+VK_LAYER_EXPORT VkFlags GetLayerOptionFlags(string option, layer_data::unordered_map<string, VkFlags> const &enum_data,
                                             uint32_t option_default) {
     VkDebugReportFlagsEXT flags = option_default;
     string option_list = layer_config.GetOption(option.c_str());
@@ -168,6 +187,7 @@ ConfigFile::ConfigFile() : file_is_parsed_(false) {
     value_map_["khronos_validation.debug_action"] = "VK_DBG_LAYER_ACTION_DEFAULT,VK_DBG_LAYER_ACTION_LOG_MSG";
 #endif  // WIN32
     value_map_["khronos_validation.log_filename"] = "stdout";
+    value_map_["khronos_validation.fine_grained_locking"] = "true";
 }
 
 const char *ConfigFile::GetOption(const string &option) {
@@ -177,10 +197,11 @@ const char *ConfigFile::GetOption(const string &option) {
         ParseFile(settings_file.c_str());
     }
 
-    if ((it = value_map_.find(option)) == value_map_.end())
+    if ((it = value_map_.find(option)) == value_map_.end()) {
         return "";
-    else
+    } else {
         return it->second.c_str();
+    }
 }
 
 void ConfigFile::SetOption(const string &option, const string &val) {
@@ -395,3 +416,24 @@ VK_LAYER_EXPORT void PrintMessageType(VkFlags vk_flags, char *msg_flags) {
         strcat(msg_flags, "PERF");
     }
 }
+
+// This catches before dlopen fails if the default Android-26 layers are being used and attempted to be ran on Android 25 or below
+#if defined(__ANDROID__)
+#include "android_ndk_types.h"  // get AHB_VALIDATION_SUPPORT macro
+void __attribute__((constructor)) CheckAndroidVersion();
+void CheckAndroidVersion() {
+#ifdef AHB_VALIDATION_SUPPORT
+    string version_env = GetEnvironment("ro.build.version.sdk");
+    int target_version = atoi(version_env.c_str());
+
+    // atoi returns 0 if GetEnvironment fails and don't want false positive errors
+    if ((target_version != 0) && (target_version < 26)) {
+        LOGCONSOLE(
+            "ERROR - Targeted Android version is %d and needs to be 26 or above. Please read "
+            "https://github.com/KhronosGroup/Vulkan-ValidationLayers/blob/master/BUILD.md for how to build the Validation Layers "
+            "for Android 25 and below",
+            target_version);
+    }
+#endif  // AHB_VALIDATION_SUPPORT
+}
+#endif  // defined(__ANDROID__)
